﻿using System;
using System.ComponentModel.Composition;
using Granta.HybridBase;

#region Steps to create a hybrid model

// Make sure you have a reference to both:
// System.ComponentModel.Composition (part of .Net framework)
// and
// Granta.HybridBase (found in the Selector install folder, eg c:\program files\CES Selector 2010)
//
// Create two public classes:
// * One to hold source material values (the 'Source Material')
// * One to return calculated values (the 'Model')
//
// To deploy your model, drop the built library (dll) into the plugins folder in the CES install folder.
// CES will automatically find your model and offer it as an option in the synthesizer user interface.

#endregion

namespace UserModel
{
    #region List of all properties used in model calculations

    // You must create a public class to store source material data for your synthesis.
    // To specify how these properties map to the database, mark up data values with the [Data] attribute, 
    // which requires a name and optional unit.

    #endregion

    public class SourceData
    {
        [Data("Density", "kg/m^3")]
        public double Density;

        [Data("Young's Modulus", "GPa")]
        public double YoungsModulus;
    }

    #region Model name, images and description

    // In order to calculated a synthesis, you must create a public class that defines your model.
    // Mark the class with the [Export("Granta.HybridModel")] attribute
    // Add the [Model] attribute to provide a name and description.

    #endregion

    [Export("Granta.HybridModel")]
    [Model("Simple Model (C#)", "A simple example model.")]
    [Image("UserModel.model.png")]
    [ModelGroup("Examples", "UserModel.thumbnail.png")]
    public class ExampleModel
    {

        #region Details of Source Material inputs

        // Create one or more instances of the source material.
        // Make sure the materials are marked as public.
        // Use the [Material] attribute give the source material a name.
        // Here we create two materials  - a matrix and a reinforcement.

        #endregion

        [Material("Matrix", "Matrix material")]
        public SourceData matrix;

        [Material("Reinforcement", "Reinforcement material")]
        public SourceData reinforcement;

        #region Details of Model Variables and Model Parameter inputs

        // Variables are of type public double, and must be marked with the 
        // [Variable] attribute, which requires a name (used in the UI) and unit.
        // 
        // You can optionally specify default values by using either the 
        // [RangeValues] or [Values] attributes.
        // 
        // You can optionally specify an upper or lower bound by using the 
        // [Bounds] attribute

        #endregion

        [Variable("Reinforcement percentage", "%", "Specify the volume fraction")]
        [RangeValues(Start = 10, End = 70, Number = 7, Logarithmic = false)]
        [Bounds(0, 70)]
        public double percentage;

        #region  Model Calculations

        // Create a method for each attribute you wish to calculate.
        // Each method should be public and return a double.
        //
        // Mark the method with the 
        // [CalculatedData] attribute, specifying the name and unit
        // of the data value you wish to calculate.
        // 
        // You can add as many methods as you require.

        #endregion

        [CalculatedData("Density", "kg/m^3")]
        public double Density()
        {
            return (percentage / 100.0) * reinforcement.Density + (1 - percentage / 100) * matrix.Density;
        }

        [CalculatedData("Young's Modulus", "GPa")]
        public double YoungsModulus()
        {
            return (percentage / 100.0) * reinforcement.YoungsModulus + (1 - percentage / 100) * matrix.YoungsModulus;
        }
    }
}
